/*
    $Id: dimm.cpp 95 2010-02-28 16:07:27Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cmath>

#include "dimm.h"

dimm::dimm_vars_type dimm::correction( const dimm_vars_type& vars ) {
	dimm_vars_type ret( vars );

	ret(1) = ret(1)*ret(1) - ret(3)*ret(3); // CCD noise removal
	value_type rho = ret(2)/ret(1);
	ret(1) = 1.15*ret(1) - 0.15*ret(2); // exposure correction
	ret(2) = rho;
	
	return ret;
}

dimm::value_type dimm::turbulence( value_type scale, value_type Knumber, value_type variance ) {
	value_type ret = variance;
	ret *= scale * scale / 16.7 / Knumber;
	return ret;
}

dimm::value_type dimm::seeing( value_type turbulence_power ) {
	value_type ret = 2.0e7 * std::pow( turbulence_power, 0.6 );
	return ret;
}

dimm_storage::dimm_storage() {
	m_storage.reserve(30);
}
dimm_storage::dimm_storage( const dimm_storage& storage ): m_storage(storage.m_storage) {
}
dimm_storage& dimm_storage::operator=( const dimm_storage& storage ) {
	dimm_storage tmp( storage );
	tmp.swap( *this );
	return *this;
}
void dimm_storage::swap( dimm_storage& storage ) throw() {
	m_storage.swap(storage.m_storage);
}
dimm_storage::~dimm_storage() {
}
void dimm_storage::accumulate( const time_type& time, unsigned int framenum, const dimm_flux_type& left, const dimm_flux_type& right, const dimm_vars_type& xvars, const dimm_flux_type& yvars ) {
	//FIXME: check datetime ordering...
	m_storage.push_back( dimm_sample( time, framenum, left, right, xvars, yvars ) );
}
void dimm_storage::correct_low_vars( ) {
	double meanx = 0;
	double meany = 0;
	for( samples_list_type::const_iterator it = m_storage.begin(); it != m_storage.end(); ++it ) {
		meanx += it->xvars_(0) / m_storage.size();
		meany += it->yvars_(0) / m_storage.size();
	}
	for( samples_list_type::iterator it = m_storage.begin(); it != m_storage.end(); ++it ) {
		it->xvars_(1) += ( it->xvars_(0) - meanx ) * ( it->xvars_(0) - meanx );
		it->yvars_(1) += ( it->yvars_(0) - meany ) * ( it->yvars_(0) - meany );
	}
}
