/*
    $Id: integrals.h 90 2010-02-25 18:29:49Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INTEGRALS_H
#define _INTEGRALS_H

#include <string>
#include <stdexcept>

#include <boost/tuple/tuple.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "nocase_less.h"

#include "typesdef.h"
#include "weif.h"

using namespace boost::numeric::ublas;

class integrals_factory;
class integrals_functor;

class integrals_factory {
	public:
		typedef matrix< double > dcf_matrix_type;
		typedef std::map< std::string, std::pair< dcf_matrix_type, double >, nocase_less< std::string > > storage_type;
		typedef integrals_functor functor_type;
		typedef storage_type::const_iterator const_iterator;
	private:
		storage_type m_storage;

		dcf_matrix_type calculate( const std::string& spectral_type, const Weights& awf );
	public:
		integrals_factory();
		integrals_factory( const integrals_factory& factory );
		void swap( integrals_factory& factory ) throw();
		integrals_factory& operator=( const integrals_factory& factory );
		~integrals_factory();

		functor_type get_functor( const std::string& spectral_type, const Weights& awf );
};

class integrals_functor {
	public:
		typedef integrals_factory             factory_type;
		typedef factory_type::dcf_matrix_type dcf_matrix_type;

		struct atmospheric_struct {
			double fseeing;
			double isangle;
			double altieff;
			double ticonst;
			double moment0;
			double moment2;

			inline const double free_seeing()   const { return fseeing; }
			inline const double isoplanatic_angle() const { return isangle; }
			inline const double altitude_eff()  const { return altieff; }
			inline const double time_constant() const { return ticonst; }
			inline const double moment_0() const { return moment0; }
			inline const double moment_2() const { return moment2; }
		};
		typedef atmospheric_struct result_type;

		typedef symmetric_matrix<double> indices_type;
		typedef vector<double> indesis_type;
	private:
		const dcf_matrix_type& m_dcf;
		double m_lambda_eff;
	public:
		integrals_functor( const dcf_matrix_type& dcf, double lambda_eff ):
			m_dcf( dcf ), m_lambda_eff( lambda_eff ) {}

		result_type operator() ( const indices_type& indices, const indesis_type& indesis, double mz ) const;
};

#endif // _INTEGRALS_H
